#!/usr/bin/env python

'''test_convert_xl.py - tests the convert_xl module
Chris R. Coughlin (TRI/Austin, Inc.)
'''

from podbrowser.util import convert_xl
import xlrd
import os.path
import os
import unittest

class TestConvertXL(unittest.TestCase):
    '''Tests the ConvertXLFile class'''

    def setUp(self):
        test_file = os.path.abspath(__file__)
        self.test_path = os.path.dirname(test_file)
        self.samplefilename = os.path.join(self.test_path, "agnesi_matrix.xls")
        self.xl_converter = convert_xl.ConvertExcelFile(self.samplefilename)

    def test_creation(self):
        '''Verify creation of the class'''
        self.assertEqual(self.samplefilename, self.xl_converter.xlfile)
        self.assertEqual(self.xl_converter.delim, ',')
        self.assertIsInstance(self.xl_converter.workbook, xlrd.Book)

    def test_nofilefound(self):
        '''Verify IOError raised if spreadsheet not found'''
        with self.assertRaises(IOError):
            xl_converter = convert_xl.ConvertExcelFile('no file')

    def test_read_workbook(self):
        '''Verify reading the entire workbook'''
        sheet_names, workbook_contents = self.xl_converter.read_workbook()
        contents = [self.xl_converter.read_sheet(sheet) for sheet in sheet_names]
        self.assertEqual(sheet_names, self.xl_converter.workbook.sheet_names())

    def test_get_workbook(self):
        '''Verify retrieval of an Excel spreadsheet'''
        workbook = self.xl_converter.get_workbook(self.samplefilename)
        self.assertIsInstance(workbook, xlrd.Book)
        # Ensure some basics about an Excel spreadsheet match
        self.assertEqual(workbook.nsheets, self.xl_converter.workbook.nsheets)
        self.assertEqual(workbook.user_name, self.xl_converter.workbook.user_name)
        self.assertListEqual(workbook.name_obj_list, self.xl_converter.workbook.name_obj_list)
        self.assertListEqual(workbook.sheet_names(), self.xl_converter.workbook.sheet_names())

    def test_read_sheet(self):
        '''Verify retrieval of a named sheet'''
        sheet_contents = list(self.xl_converter.read_sheet("agnesi_matrix"))
        # Try a sample of the spreadsheet
        row_nine = sheet_contents[8]
        row_str = [str(cell) for cell in row_nine]
        first_cell = row_str[0]
        rest_of_row = row_str[1:]
        expected_first_cell = '# Detrending function:  None'
        self.assertEqual(first_cell, expected_first_cell)
        for cell in rest_of_row:
            self.assertEqual(cell, '')

    def test_convert_sheet(self):
        '''Verify export of a named sheet'''
        export_fname = os.path.join(self.test_path, "agnesi_matrix_sheetconversion.csv")
        self.xl_converter.convert_sheet("agnesi_matrix", export_fname)
        self.assertTrue(os.path.exists(export_fname))
        self.assertNotEqual(os.path.getsize(export_fname), 0)
        if os.path.exists(export_fname):
            os.remove(export_fname)

    def test_convert_workbook(self):
        '''Verify export of an entire Excel spreadsheet'''
        export_fname = os.path.join(self.test_path, "agnesi_matrix_conversion.csv")
        self.xl_converter.convert_workbook(export_fname)
        self.assertTrue(os.path.exists(export_fname))
        self.assertNotEqual(os.path.getsize(export_fname), 0)
        if os.path.exists(export_fname):
            os.remove(export_fname)

    def test_get_col(self):
        '''Verify retrieving a specified column from the spreadsheet'''
        col_num = 5
        expected_col_contents = ['', '', '', '', '', '', '', '', '', -3.39E+00 , -3.35E+00, -3.32E+00, -3.27E+00, 
            -3.23E+00, -3.18E+00, -3.13E+00, -3.07E+00, -3.00E+00, -2.93E+00, -2.85E+00, -2.76E+00,
            -2.66E+00, -2.56E+00, -2.45E+00, -2.33E+00, -2.20E+00, -2.07E+00, -1.93E+00, -1.80E+00, 
            -1.68E+00, -1.58E+00, -1.49E+00, -1.43E+00, -1.39E+00, -1.40E+00, -1.43E+00, -1.49E+00, 
            -1.58E+00, -1.69E+00, -1.81E+00, -1.93E+00, -2.06E+00, -2.19E+00, -2.32E+00, -2.44E+00, 
            -2.56E+00, -2.67E+00, -2.76E+00, -2.85E+00, -2.93E+00, -3.00E+00, -3.07E+00, -3.13E+00, 
            -3.18E+00, -3.23E+00, -3.27E+00, -3.31E+00, -3.35E+00, -3.39E+00]
        retrieved_col_contents = self.xl_converter.get_col("agnesi_matrix", col_num)
        for idx, val in enumerate(retrieved_col_contents):
            if isinstance(val, float):
                self.assertAlmostEqual(val, expected_col_contents[idx], delta=abs(0.1*val))
            else:
                self.assertEqual(val, expected_col_contents[idx])

    def test_get_row(self):
        '''Verify retrieving a specified row from the spreadsheet'''
        row_num = 33
        expected_row_contents = [-2.01E+00, -1.89E+00, -1.76E+00, -1.64E+00, -1.52E+00, -1.39E+00, -1.27E+00,
            -1.15E+00, -1.03E+00, -9.04E-01, -7.81E-01, -6.59E-01, -5.36E-01, -4.13E-01, -2.91E-01, -1.68E-01,
            -4.57E-02, 7.68E-02, 1.99E-01, 3.22E-01, 4.44E-01, 5.67E-01, 6.89E-01, 8.12E-01, 9.34E-01, 1.06E+00,
            1.18E+00, 1.30E+00, 1.42E+00, 1.55E+00, 1.67E+00, 1.79E+00, 1.91E+00, 2.04E+00, 2.16E+00, 2.28E+00,
            2.40E+00, 2.53E+00, 2.65E+00, 2.77E+00, 2.89E+00, 3.01E+00, 3.14E+00, 3.26E+00, 3.38E+00, 3.50E+00,
            3.62E+00, 3.75E+00, 3.87E+00, 3.99E+00]
        retrieved_row_contents =  self.xl_converter.get_row("agnesi_matrix", row_num)
        for idx, val in enumerate(retrieved_row_contents):
            if isinstance(val, float):
                self.assertAlmostEqual(val, expected_row_contents[idx], delta=abs(0.1*val))
            else:
                self.assertEqual(val, expected_row_contents[idx])