#!/usr/bin/env python

'''test_finddata.py - tests the finddata module (POD Browser)
Chris R. Coughlin (TRI/Austin, Inc.)
'''

import podbrowser
from podbrowser.util import finddata
import os.path
import os
import unittest

class TestDataLocations(unittest.TestCase):
    '''Tests the DataLocations class'''

    def setUp(self):
        pass

    def test_basepath(self):
        '''Verify creation of data locations if basepath provided'''
        datafolder = '.'
        dataloc = finddata.DataLocations(datafolder)
        self.assertEqual(datafolder, dataloc.data_folder)

    def test_nobasepath(self):
        '''Verify exception if no basepath provided'''
        with self.assertRaises(TypeError):
            dataloc = finddata.DataLocations()

    def test_nomethods(self):
        '''Verify creation of standard NDE methods if no list provided'''
        datafolder = os.path.join(os.path.dirname(podbrowser.__file__), 'data')
        dataloc = finddata.DataLocations(datafolder)
        cwd = os.getcwd() # Temporarily switch folders for os.listdir call
        os.chdir(datafolder)
        methods = [entry for entry in os.listdir(datafolder) if os.path.isdir(entry)]
        os.chdir(cwd)
        self.assertListEqual(dataloc.methods, methods)
    
    def test_methods(self):
        '''Verify creation of standard NDE methods if no list provided'''
        datafolder = '.'
        ndemethods = ['Alpha', 'Beta', 'Gamma']
        dataloc = finddata.DataLocations(datafolder, methods=ndemethods)
        self.assertListEqual(dataloc.methods, ndemethods)

    def test_types(self):
        '''Verify members are correct type'''
        datafolder = '.'
        dataloc = finddata.DataLocations(datafolder)
        self.assertIsInstance(dataloc.data_folder, str)
        self.assertIsInstance(dataloc.methods, list)
        self.assertIsInstance(dataloc.folders, dict)

    def test_populate_posix(self):
        '''Verify creation of data folder names in POSIX environments'''
        datafolder = '/POD Browser/data'
        ndemethods = ['Alpha', 'Beta', 'Gamma']
        dataloc = finddata.DataLocations(datafolder, methods=ndemethods)
        for method in ndemethods:
            self.assertEqual(dataloc.folders[method], os.path.join(datafolder, method))

    def test_populate_windows(self):
        '''Verify creation of data folder names in Windows environments'''
        datafolder = '\\My Documents\\POD Browser\\data'
        ndemethods = ['Alpha', 'Beta', 'Gamma']
        dataloc = finddata.DataLocations(datafolder, methods=ndemethods)
        for method in ndemethods:
            self.assertEqual(dataloc.folders[method], os.path.join(datafolder, method))
    
    def test_method_folder(self):
        '''Verify retrieval of full path of method folder'''
        method_name = "Magnetic Particle"
        data_folder = os.path.join(os.path.dirname(podbrowser.__file__), 'data')
        method_folder = os.path.join(data_folder, method_name)
        dataloc = finddata.DataLocations(data_folder)
        self.assertEqual(method_folder, dataloc.method_folder("Magnetic Particle"))

class TestData(unittest.TestCase):
    '''Tests the Data class'''

    def setUp(self):
        self.datapath = os.path.join(os.path.dirname(podbrowser.__file__), 'data')
        self.data_loc = finddata.DataLocations(self.datapath)
        self.methodpath = self.data_loc.method_folder("Magnetic Particle")
        self.data = finddata.Data(self.methodpath)

    def test_creation(self):
        '''Verifies creation and proper types'''
        path = '.'
        data = finddata.Data(path)
        self.assertEqual(path, data.folder)
        self.assertIsInstance(data.inspections, list)
        self.assertIsInstance(data.datasets, dict)

    def test_populate_inspections(self):
        '''Verify populating the list of inspections'''
        self.data.populate_inspections()
        inspections = []
        for root, dirs, files in os.walk(self.methodpath):
            for dname in dirs:
                dpath = os.path.join(root, dname)
                inspections.append(dpath)
        self.assertItemsEqual(inspections, self.data.inspections)

    def test_populate_data(self):
        '''Verify populating the dict of data'''
        datasets = {}
        self.data.populate_inspections()
        self.data.populate_data()
        for inspection in self.data.inspections:
            inspection_data = []
            for root, dirs, files in os.walk(inspection):
                for fname in files:
                    inspection_data.append(os.path.join(root, fname))
            datasets[os.path.relpath(inspection, self.methodpath)] = inspection_data
        self.assertDictEqual(datasets, self.data.datasets)

    def test_populate(self):
        '''Verify populating list of inspections and dict of data'''
        inspections = []
        datasets = {}
        self.data.populate()
        for root, dirs, files in os.walk(self.methodpath):
            for dname in dirs:
                dpath = os.path.join(root, dname)
                inspections.append(dpath)
        self.assertItemsEqual(inspections, self.data.inspections)
        for inspection in self.data.inspections:
            inspection_data = []
            for root, dirs, files in os.walk(inspection):
                for fname in files:
                    inspection_data.append(os.path.join(root, fname))
            datasets[os.path.relpath(inspection, self.methodpath)] = inspection_data
        self.assertDictEqual(datasets, self.data.datasets)

    def test_find_dataset(self):
        '''Verify retrieval of the full path and filename of a data file'''
        self.data.populate()
        datapath = os.path.join(self.methodpath, 'BMT1', 'B-MT1L', 'B1000(2L.XLS')
        self.assertEqual(datapath, self.data.get_data_fname('BMT1', 'B1000(2L.XLS'))
        self.assertIsNone(self.data.get_data_fname('BMT1', 'NO_SUCH_DATA_FILE'))
        self.assertIsNone(self.data.get_data_fname('NO_SUCH_DATASET', 'B1000(2L.XLS'))
        # Should also retrieve same data file for a child dataset
        self.assertEqual(datapath, self.data.get_data_fname(['BMT1', 'B-MT1L'], 'B1000(2L.XLS'))
        self.assertIsNone(self.data.get_data_fname(['BMT1', 'B-MT1L'], 'NO_SUCH_DATA_FILE'))
