#!/usr/bin/env python

'''plot_panel.py - simple wxPython matplotlib window for plotting POD data
Chris R. Coughlin (TRI/Austin, Inc.)
'''

import wx
import os.path
import matplotlib
from matplotlib.backends.backend_wxagg import NavigationToolbar2WxAgg, FigureCanvasWxAgg as FigureCanvas
import matplotlib.figure
from util import program_info

class PlotPanel(wx.Frame):
    '''Simple wxPython Frame to display a matplotlib figure for POD & hit/miss data'''
    widget_margin = 3.0 # Default border around widgets
    label_pct = 0.25 # Default resizing factor for labels - 25%
    ctrl_pct = 1.0 # Default resizing factor for controls - 100%
    sizer_flags = wx.ALL | wx.EXPAND # Default resizing flags for controls

    def __init__(self, pod_data, hm_data, parent=None, fname=None):
        wx.Frame.__init__(self, id=wx.ID_ANY, name='', parent = parent, size=wx.Size(640, 480))
        self.SetIcon(parent.GetIcon())
        self.pod_data = pod_data
        self.hm_data = hm_data
        self.fname = fname
        if self.fname is None:
            self.SetTitle(u'POD Plot')
        else:
            self.SetTitle(u'POD Plot:  {0}'.format(os.path.basename(self.fname)))
        self.init_ui()
        self.plot_pod()

    def init_ui(self):
        '''Creates and lays out the user interface'''
        self.main_panel = wx.ScrolledWindow(self, wx.ID_ANY)
        self.main_panel.SetScrollbars(1, 1, 1, 1)
        self.main_panel_sizer = wx.BoxSizer(wx.VERTICAL)
        self.init_ctrls()
        self.main_panel.SetSizerAndFit(self.main_panel_sizer)
        self.SetMinSize(self.main_panel.GetBestSize())

    def init_ctrls(self):
        '''Initializes the controls for the Frame'''
        self.figure = matplotlib.figure.Figure()
        self.canvas = FigureCanvas(self.main_panel, -1, self.figure) 
        self.toolbar = NavigationToolbar2WxAgg(self.canvas)
        self.size_plot()
        self.axes = self.figure.add_subplot(111, navigate=True)
        self.axes.grid(True)
        self.main_panel_sizer.Add(self.canvas, self.ctrl_pct, self.sizer_flags, self.widget_margin)
        self.main_panel_sizer.Add(self.toolbar, 0, self.sizer_flags, self.widget_margin)
        self.toolbar.Realize()

    def plot_pod(self):
        '''Creates the POD and hit/miss plots'''
        self.axes.plot(self.pod_data[0], self.pod_data[1], 'b-', label='Predicted POD', linewidth=2)
        self.axes.plot(self.hm_data[0], self.hm_data[1], 'ro', label='Hit/Miss Data')
        self.axes.grid(True)
        self.axes.set_ylim(0, 100)
        if self.fname is None:
            title = "Probability Of Detection"
        else:
            title = "Probability Of Detection - {0}".format(os.path.basename(self.fname))
        self.figure.suptitle(title, fontsize=14, fontweight='semibold')
        self.axes.set_title('Generated By PODBrowser v.{0}'.format(program_info.version), fontsize=8,
            color='#4f6581')
        #self.axes.set_title(title, fontsize=14, fontweight='bold')
        self.axes.set_xlabel("Defect Size", fontsize=12, fontweight='roman')
        self.axes.set_ylabel("Probability Of Detection (%)", fontsize=12, fontweight='roman')
        legend = self.axes.legend(loc='lower right', fancybox=True)
        legend.get_frame().set_alpha(0.5)
        self.refresh_plot()

    def refresh_plot(self):
        '''Redraws the plot canvas'''
        self.canvas.draw()

    def size_plot(self):
        '''Resizes canvas and plot to fit nicely in the confines of the window'''
        scale_factor = 0.75
        pixels = (scale_factor*self.GetClientSize()[0], scale_factor*self.GetClientSize()[1])
        self.canvas.SetSize(pixels)
        self.figure.set_size_inches(float(pixels[0])/self.figure.get_dpi(), float(pixels[1] )/self.figure.get_dpi() )