#!/usr/bin/env python

''' podbrowser_ui.py - wxPython user interface for PODBrowser
Chris R. Coughlin (TRI/Austin, Inc.)
'''

import sys
import webbrowser
import glob
import os.path
import wx
import wx.lib.dialogs
import warnings
import icon
import text_display
import preview_file
import about_dlg
from util import program_info
from util import finddata
from util import open_file

class UI(wx.Frame):
    '''The primary user interface'''
    widget_margin = 3.0 # Default border around widgets
    lbl_pct = 0.25 # Default resizing factor for labels - 25%
    ctrl_pct = 1.0 # Default resizing factor for controls - 100%
    sizer_flags = wx.ALL | wx.EXPAND # Default resizing flags for controls
    
    def __init__(self, parent, datafolder):
        _size = wx.Size(800, 600)
        self.parent = parent
        wx.Frame.__init__(self, id=wx.ID_ANY, name='', parent=None,
            size=_size, title="PODBrowser")
        if 'win32' in sys.platform:
            self.SetIcon(wx.Icon(os.path.join(
                'bitmaps','win32icon64x64.ico'), wx.BITMAP_TYPE_ICO))
        else:
            self.SetIcon(icon.geticonIcon())
        self.version = program_info.version
        self.datafolder = datafolder
        self.init_data()
        self.init_menu()
        self.init_ui()
        self.inspection = None # List of data for a given NDE Technique and selected inspection
        self.data_file = None # Full path and filename of selected data file

    def init_data(self):
        '''Scans for POD Data'''
        self.pod_data = finddata.DataLocations(self.datafolder)

    def init_menu(self):
        '''Creates the UI's application menu'''
        self.menubar = wx.MenuBar()
        self.init_file_menu()
        self.init_help_menu()
        self.SetMenuBar(self.menubar)

    def init_file_menu(self):
        '''Creates the File menu'''
        self.file_mnu = wx.Menu() # File menu
        quit_mnui = wx.MenuItem(self.file_mnu, wx.ID_ANY, text = "E&xit\tCTRL+X",
            help = "Exit The Program")
        self.file_mnu.AppendItem(quit_mnui)
        self.Bind(wx.EVT_MENU, self.on_quit, id=quit_mnui.GetId())
        self.Bind(wx.EVT_CLOSE, self.on_quit)
        self.menubar.Append(self.file_mnu,"&File")

    def init_help_menu(self):
        '''Creates the Help menu'''
        self.help_mnu = wx.Menu() # Help menu
        tutorial_mnui = wx.MenuItem(self.help_mnu, wx.ID_ANY, text="Using PODBrowser",
            help="Help on PODBrowser usage and terms")
        self.help_mnu.AppendItem(tutorial_mnui)
        self.Bind(wx.EVT_MENU, self.on_tutorial, id=tutorial_mnui.GetId())
        disclaimer_mnui = wx.MenuItem(self.help_mnu, wx.ID_ANY, text='Disclaimers',
            help='Important information about using the POD Databook')
        self.help_mnu.AppendItem(disclaimer_mnui)
        self.Bind(wx.EVT_MENU, self.on_disclaimer, id=disclaimer_mnui.GetId())
        license_mnui = wx.MenuItem(self.help_mnu, wx.ID_ANY, text='License',
            help='Licensing Information')
        self.help_mnu.AppendItem(license_mnui)
        self.Bind(wx.EVT_MENU, self.on_license, id=license_mnui.GetId())
        credits_mnui = wx.MenuItem(self.help_mnu, wx.ID_ANY, text='Additional Credits',
            help='Additional Credits')
        self.help_mnu.AppendItem(credits_mnui)
        self.Bind(wx.EVT_MENU, self.on_credits, id=credits_mnui.GetId())
        self.help_mnu.AppendSeparator()
        about_mnui = wx.MenuItem(self.help_mnu, wx.ID_ANY, text='About This Program...',
            help='About the POD Databook Browser')
        self.help_mnu.AppendItem(about_mnui)
        self.Bind(wx.EVT_MENU, self.on_about, id=about_mnui.GetId())
        about_tri_mnui = wx.MenuItem(self.help_mnu, wx.ID_ANY, text='About TRI/Austin...',
            help="About TRI/Austin's NDE Division")
        self.help_mnu.AppendItem(about_tri_mnui)
        self.Bind(wx.EVT_MENU, self.on_about_tri, id=about_tri_mnui.GetId())
        about_nditoolbox_mnui = wx.MenuItem(self.help_mnu, wx.ID_ANY, text='About NDIToolbox...',
            help="About TRI's NDIToolbox")
        self.help_mnu.AppendItem(about_nditoolbox_mnui)
        self.Bind(wx.EVT_MENU, self.on_about_nditoolbox, id=about_nditoolbox_mnui.GetId())
        self.menubar.Append(self.help_mnu,"&Help")

    def init_ui(self):
        '''Lays out the UI's structure'''
        self.main_panel_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.init_controls()
        self.SetSizerAndFit(self.main_panel_sizer)
        self.status_bar = wx.StatusBar(self, -1)
        self.SetStatusBar(self.status_bar)
        self.SetStatusText("POD Data Browser {0} Build {1}".format(self.version, program_info.build_date), 0)

    def init_controls(self):
        '''Creates and adds the controls to the UI'''
        self.create_technique_panel()
        self.create_inspection_panel()
        self.create_dataset_panel()
        self.ctrl_panel = wx.Panel(self)
        ctrl_panel_sizer = wx.BoxSizer(wx.VERTICAL)
        ctrl_panel_lbl = wx.StaticText(self.ctrl_panel, wx.ID_ANY, "", wx.DefaultPosition,
            wx.DefaultSize)
        ctrl_panel_sizer.Add(ctrl_panel_lbl, 0, self.sizer_flags, self.widget_margin)
        self.previewdata_btn = wx.Button(self.ctrl_panel, wx.ID_ANY, "Preview Data", wx.DefaultPosition,
            wx.DefaultSize)
        self.previewdata_btn.SetToolTipString(u'Previews the selected POD data file')
        self.Bind(wx.EVT_BUTTON, self.on_previewbtn_click, self.previewdata_btn)
        ctrl_panel_sizer.Add(self.previewdata_btn, 0, self.sizer_flags, self.widget_margin)
        self.opendata_btn = wx.Button(self.ctrl_panel, wx.ID_ANY, "Open Data", wx.DefaultPosition,
            wx.DefaultSize)
        self.opendata_btn.SetToolTipString(u'Opens the selected POD data file')
        ctrl_panel_sizer.Add(self.opendata_btn, 0, self.sizer_flags, self.widget_margin)
        self.Bind(wx.EVT_BUTTON, self.on_openbtn_click, self.opendata_btn)
        self.ctrl_panel.SetSizerAndFit(ctrl_panel_sizer)
        self.main_panel_sizer.Add(self.ctrl_panel, 0, self.sizer_flags, 0)

    def create_technique_panel(self):
        '''Creates the Technique panel - list of inspection techniques'''
        self.technique_panel = wx.Panel(self)
        technique_panel_sizer = wx.BoxSizer(wx.VERTICAL)
        technique_panel_lbl = wx.StaticText(self.technique_panel, wx.ID_ANY, "Technique", wx.DefaultPosition,
            wx.DefaultSize)
        technique_panel_sizer.Add(technique_panel_lbl, self.lbl_pct, self.sizer_flags, self.widget_margin)
        self.technique_lb = wx.ListBox(self.technique_panel, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize,
            choices=sorted(self.pod_data.methods, key=str.lower), style=wx.LB_SINGLE)
        self.technique_lb.SetToolTipString(u'Lists the inspection techniques found in the data folder')
        self.Bind(wx.EVT_LISTBOX, self.on_lb_select, self.technique_lb)
        technique_panel_sizer.Add(self.technique_lb, self.ctrl_pct, self.sizer_flags, self.widget_margin)
        self.technique_panel.SetSizerAndFit(technique_panel_sizer)
        self.main_panel_sizer.Add(self.technique_panel, 1, self.sizer_flags, 0)

    def create_inspection_panel(self):
        '''Creates the Inspection panel - list of inspections for a selected technique'''
        self.inspection_panel = wx.Panel(self)
        inspection_panel_sizer = wx.BoxSizer(wx.VERTICAL)
        inspection_panel_lbl = wx.StaticText(self.inspection_panel, wx.ID_ANY, "Inspection", wx.DefaultPosition,
            wx.DefaultSize)
        inspection_panel_sizer.Add(inspection_panel_lbl, self.lbl_pct, self.sizer_flags, self.widget_margin)
        self.inspection_lb = wx.ListBox(self.inspection_panel, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize,
            style=wx.LB_SINGLE)
        self.inspection_lb.SetToolTipString(u'Lists the specific inspections found for the selected technique')
        self.Bind(wx.EVT_LISTBOX, self.on_lb_select, self.inspection_lb)
        inspection_panel_sizer.Add(self.inspection_lb, self.ctrl_pct, self.sizer_flags, self.widget_margin)
        self.inspection_panel.SetSizerAndFit(inspection_panel_sizer)
        self.main_panel_sizer.Add(self.inspection_panel, 1, self.sizer_flags, 0)

    def create_dataset_panel(self):
        '''Creates the Datasets panel - list of inspection datasets'''
        self.dataset_panel = wx.Panel(self)
        dataset_panel_sizer = wx.BoxSizer(wx.VERTICAL)
        dataset_panel_lbl = wx.StaticText(self.dataset_panel, wx.ID_ANY, "Datasets", wx.DefaultPosition,
            wx.DefaultSize)
        dataset_panel_sizer.Add(dataset_panel_lbl, self.lbl_pct, self.sizer_flags, self.widget_margin)
        self.dataset_lb = wx.ListBox(self.dataset_panel, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize,
            style=wx.LB_SINGLE)
        self.dataset_lb.SetToolTipString(u'Lists the specific data files found for the selected inspection')
        self.Bind(wx.EVT_LISTBOX, self.on_lb_select, self.dataset_lb)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.on_dataset_lb_dclick, self.dataset_lb)
        dataset_panel_sizer.Add(self.dataset_lb, self.ctrl_pct, self.sizer_flags, self.widget_margin)
        self.dataset_panel.SetSizerAndFit(dataset_panel_sizer)
        self.main_panel_sizer.Add(self.dataset_panel, 1, self.sizer_flags, 0)

    # Event Handlers
    def on_quit(self, evt):
        '''Handles the Quit event'''
        msg = "Are you sure you want to quit?"
        confirmquit_dlg = wx.MessageDialog(self, msg,
            'Exit PODBrowser?', wx.OK | wx.CANCEL | wx.ICON_QUESTION)
        if confirmquit_dlg.ShowModal() == wx.ID_OK:
            self.Destroy()

    def on_about(self, evt):
        '''Handles the About event'''
        about_dlg.AboutDialog(self, -1, version=self.version)

    def on_disclaimer(self, evt):
        '''Handles the POD Disclaimer menu event by loading the disclaimer file'''
        disclaimer_path = os.path.join(sys.path[0], 'docs', 'disclaimer.txt')
        if os.path.exists(disclaimer_path):
            disclaimer_dlg = text_display.TextDisplay(textfile=disclaimer_path, parent=self, title='Disclaimer')
            disclaimer_dlg.Show()
        else:
            msg = "Unable to load disclaimer, please refer to docs/disclaimer.txt for more information."
            err_dlg = wx.MessageDialog(parent = self, message = msg,
                caption = "Unable To Load Disclaimer", style = wx.ICON_EXCLAMATION)
            err_dlg.ShowModal()

    def on_license(self, evt):
        '''Handles the License menu event by loading the license file'''
        license_path = os.path.join(sys.path[0], 'docs', 'license.txt')
        if os.path.exists(license_path):
            license_dlg = text_display.TextDisplay(textfile=license_path, parent=self, title='PODBrowser License')
            license_dlg.Show()   
        else:
            msg = "Unable to load license, please refer to docs/license.txt for more information."
            err_dlg = wx.MessageDialog(parent = self, message = msg,
                caption = "Unable To Load License", style = wx.ICON_EXCLAMATION)
            err_dlg.ShowModal()

    def on_credits(self, evt):
        '''Handles the Additional Credits menu event by detailing the contributions'''
        creds_txt = ["TRI would like to thank the following for their contributions:",
            "John Machin - xlrd Excel data extraction module - www.lexicon.net/sjmachin/xlrd.htm",
            "Chris Coughlin - wxspreadsheet_demo - https://bitbucket.org/ccoughlin",
            "StackOverflow community - www.stackoverflow.com",
            "matplotlib project - matplotlib.sourceforge.net",
            "wxPython project - www.wxpython.org",
            'Python Software Foundation - www.python.org']
        creds_dlg = wx.lib.dialogs.ScrolledMessageDialog(self, '\n\n'.join(creds_txt), "Additional Credits & Thanks")
        creds_dlg.SetIcon(self.GetIcon())
        creds_dlg.SetSize(creds_dlg.GetBestSize())
        creds_dlg.ShowModal()

    def on_tutorial(self, evt):
        '''Handles the menu request for opening the tutorial in the user's default web browser'''
        tutorial_path = os.path.join(sys.path[0], 'docs', 'podbrowser.html')
        if len(glob.glob(tutorial_path)) != 0:
            self.open_url(tutorial_path)
        else:
            msg = "Unable to load the tutorial, please refer to docs/podbrowser.html for more information.'"
            err_dlg = wx.MessageDialog(parent = self, message = msg,
                caption = "Unable To Find Tutorial", style = wx.ICON_EXCLAMATION)
            err_dlg.ShowModal()

    def on_about_tri(self, evt):
        '''Handles the About TRI event by opening the TRI page in the default webbrowser'''
        self.open_url("http://www.tri-austin.com/index.php?c=2&s=3&id=22")

    def on_about_nditoolbox(self, evt):
        '''Handles the About NDIToolbox event by opening the NDIToolbox page in the default webbrowser'''
        self.open_url("http://www.nditoolbox.com/")

    def open_url(self, url):
        '''Opens the specified URL in the user's default webbrowser.'''
        try:
            webbrowser.open(url)
        except webbrowser.Error:
            msg = "Unable to open '{0}', please contact TRI/Austin for assistance.'".format(url)
            err_dlg = wx.MessageDialog(parent = self, message = msg,
                caption = "Unable To Load Web Page", style = wx.ICON_EXCLAMATION)
            err_dlg.ShowModal()

    def on_lb_select(self, evt):
        '''Handles selection events on the listbox controls'''
        if evt.IsSelection():
            if evt.GetId() == self.technique_lb.GetId():
                self.inspection_lb.Set(sorted(self.get_inspections(evt.GetString()), key=str.lower))
                self.dataset_lb.Clear()
                self.SetStatusText('')
            elif evt.GetId() == self.inspection_lb.GetId():
                self.dataset_lb.Set(sorted(self.get_datasets(evt.GetString()), key=str.lower))
                self.SetStatusText('')
            elif evt.GetId() == self.dataset_lb.GetId():
                self.set_data_file()
                self.SetStatusText("File:  {0}".format(os.path.abspath(self.data_file)), 0)

    def on_previewbtn_click(self, evt):
        '''Handles click on Preview Data button - opens new window with first sheet in selected data'''
        if self.dataset_lb.GetSelection() != wx.NOT_FOUND:
            self.preview_data()
        else:
            err_dlg = wx.MessageDialog(parent=self, message="Please select a dataset to preview.",
                caption="No Data File Selected", style=wx.ICON_EXCLAMATION)
            err_dlg.ShowModal()

    def preview_data(self):
        '''Previews the currently selected data file.'''
        self.set_data_file()
        if self.data_file is not None:
            preview_window = preview_file.PreviewFile(parent=self, xl_file=self.data_file)
            preview_window.Show()
        else:
            err_dlg = wx.MessageDialog(parent=self, message="Please select a dataset to preview.",
                caption="No Data File Selected", style=wx.ICON_EXCLAMATION)
            err_dlg.ShowModal()

    def on_openbtn_click(self, evt):
        '''Handles click on Open Data button - opens selected data in associated application'''
        if self.dataset_lb.GetSelection() != wx.NOT_FOUND:
            self.open_data()
        else:
            err_dlg = wx.MessageDialog(parent=self, message="Please select a dataset to open.",
                caption="No Data File Selected", style=wx.ICON_EXCLAMATION)
            err_dlg.ShowModal()
    
    def on_dataset_lb_dclick(self, evt):
        '''Handles the double click event on the dataset listbox - opens selected data in associated application'''
        if self.dataset_lb.GetSelection() != wx.NOT_FOUND:
            self.open_data()
        else:
            err_dlg = wx.MessageDialog(parent=self, message="Please select a dataset to open.",
                caption="No Data File Selected", style=wx.ICON_EXCLAMATION)
            err_dlg.ShowModal()

    def open_data(self):
        '''Opens the currently selected data file with the associated application.'''
        self.set_data_file()
        if self.data_file is not None:
            # If the OS isn't POSIX, Windows, or OS X, warn the user this may not work
            if self.check_os():
                try:
                    open_file.open_file(self.data_file)
                except IOError: # File not found
                    err_dlg = wx.MessageDialog(parent=self, 
                        message="Unable to locate the data file.  Please contact TRI/Austin for assistance.",
                        caption="File Not Found", style=wx.ICON_EXCLAMATION)
                    err_dlg.ShowModal()
                except WindowsError: # No associated application on Windows
                    msg = '  '.join(['No associated application for this type of data file was found.',
                        'Please contact TRI/Austin for assistance.'])
                    err_dlg = wx.MessageDialog(parent=self, 
                        message=msg,
                        caption="No Associated Application", style=wx.ICON_EXCLAMATION)
                    err_dlg.ShowModal()
        else:
            err_dlg = wx.MessageDialog(parent=self, message="Please select a dataset to open.",
                caption="No Data File Selected", style=wx.ICON_EXCLAMATION)
            err_dlg.ShowModal()

    def check_os(self):
        '''Checks the operating system as a wrapper for some OS functions.
        Returns True if OS is POSIX, OS X, or Windows or if the user gives the ok to try it anyway'''
        retval = False
        if os.name in ('posix', 'nt', 'mac'):
            retval = True
        else:
            msg = "Warning-this feature may not be supported on this operating system.\nProceed anyway?"
            proceed_dlg = wx.MessageDialog(self, msg, 'Continue With Operation?', wx.OK | wx.CANCEL | wx.ICON_QUESTION)
            if proceed_dlg.ShowModal() == wx.ID_OK:
                retval = True
        return retval

    def get_inspections(self, technique_name):
        '''Returns a list of inspections for the specified technique_name'''
        self.inspection = finddata.Data(self.pod_data.method_folder(technique_name))
        self.inspection.populate()
        return [os.path.relpath(insp, self.inspection.folder) for insp in self.inspection.inspections]

    def get_datasets(self, inspection):
        '''Returns a list of datasets for the specified inspection'''
        return [os.path.basename(data) for data in self.inspection.datasets[inspection]]

    def set_data_file(self):
        '''Sets the full path and filename of the data file selected'''
        if self.inspection is not None:
            inspection = self.inspection_lb.GetStringSelection()
            if os.path.sep in inspection:
                inspection = inspection.split(os.path.sep)
            self.data_file = self.inspection.get_data_fname(inspection, 
                self.dataset_lb.GetStringSelection())