#!/usr/bin/env python
"""preview_file.py - loads and previews an Excel spreadsheet, based on wxspreadsheet project
https://bitbucket.org/ccoughlin/wxspreadsheet/overview

Chris R. Coughlin (TRI/Austin, Inc.)
"""

import wx
import os.path
import wxspreadsheet
import plot_panel
from util import convert_xl

class PreviewFile(wx.Frame):
    '''Simple user interface to demo the wxSpreadsheet control'''
    widget_margin = 3.0 # Default border around widgets
    label_pct = 0.25 # Default resizing factor for labels - 25%
    ctrl_pct = 1.0 # Default resizing factor for controls - 100%
    sizer_flags = wx.ALL | wx.EXPAND # Default resizing flags for controls

    def __init__(self, xl_file, parent=None):
        wx.Frame.__init__(self, id=wx.ID_ANY, name='', parent = parent)
        self.SetIcon(parent.GetIcon())
        self.xl_file = xl_file
        self.SetTitle(u"Preview:  {0}".format(os.path.basename(self.xl_file)))
        self.sheet_names = []
        try:
            self.init_conversion()
            self.init_ui()
            self.load_sheet(self.sheet_names[0])
        except convert_xl.xlrd.XLRDError:  # Bad file
            err_dlg = wx.MessageDialog(parent=self, message="This file does not appear to be a valid spreadsheet.",
                caption="Unable To Preview File", style=wx.ICON_EXCLAMATION)
            err_dlg.ShowModal()
            self.Destroy()

    def init_conversion(self):
        '''Starts the xlrd converter on this instance's Excel file'''
        self.xl_converter = convert_xl.ConvertExcelFile(self.xl_file, delim=' ')
        self.sheet_names = self.xl_converter.workbook.sheet_names()

    def init_ui(self):
        '''Creates and lays out the user interface'''
        self.main_panel = wx.Panel(self)
        self.main_panel_sizer = wx.BoxSizer(wx.VERTICAL)

        self.sheet_choice_panel = wx.Panel(self.main_panel)
        self.sheet_choice_panel_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.sheet_choice_lbl = wx.StaticText(self.sheet_choice_panel, wx.ID_ANY, u'Display Sheet:  ', wx.DefaultPosition,
            wx.DefaultSize)
        self.sheet_choice_panel_sizer.Add(self.sheet_choice_lbl, self.label_pct, wx.ALIGN_CENTER_VERTICAL, 
            self.widget_margin)
        self.sheet_choice = wx.Choice(self.sheet_choice_panel, wx.ID_ANY, choices=self.sheet_names)
        self.sheet_choice.SetToolTipString(u'Specify which sheet in the spreadsheet is displayed')
        self.sheet_choice.SetSelection(0)
        self.Bind(wx.EVT_CHOICE, self.on_sheet_choice, self.sheet_choice)
        self.sheet_choice_panel_sizer.Add(self.sheet_choice, self.label_pct, wx.ALIGN_CENTER_VERTICAL, 
            self.widget_margin)
        self.plot_btn = wx.Button(self.sheet_choice_panel, wx.ID_ANY, u'Plot POD', wx.DefaultPosition, wx.DefaultSize)
        self.plot_btn.SetToolTipString(u'Plots POD and hit/miss for the current sheet')
        self.plot_btn.SetDefault()
        self.Bind(wx.EVT_BUTTON, self.on_plot, self.plot_btn)
        self.sheet_choice_panel_sizer.Add(self.plot_btn, self.label_pct, self.sizer_flags, self.widget_margin)
        self.sheet_choice_panel.SetSizerAndFit(self.sheet_choice_panel_sizer)
        self.main_panel_sizer.Add(self.sheet_choice_panel, 0, self.sizer_flags, self.widget_margin)

        self.spreadsheet = wxspreadsheet.Spreadsheet(self.main_panel)        
        self.main_panel_sizer.Add(self.spreadsheet, 1, self.sizer_flags, self.widget_margin)
        
        self.main_panel.SetSizerAndFit(self.main_panel_sizer)
        self.SetMinSize(self.main_panel.GetBestSize())
        self.statusbar = wx.StatusBar(self, -1)
        self.SetStatusBar(self.statusbar)
        self.SetStatusText("File:  {0}".format(os.path.abspath(self.xl_file), 0))

    # Event handlers
    def on_sheet_choice(self, evt):
        '''Responds to a change in the choice of sheets - loads the sheet for display'''
        self.load_sheet(evt.GetString())

    def load_sheet(self, sheet_name):
        '''Loads and displays the specified sheet from the Excel spreadsheet'''
        self.spreadsheet.ClearGrid()
        self.spreadsheet.SetNumberCols(0)
        self.spreadsheet.SetNumberRows(0)
        sheet_contents = self.xl_converter.read_sheet(sheet_name)
        rownum = 0
        for row in sheet_contents:
            self.spreadsheet.AppendRows(1)
            numcols = len(row)
            if self.spreadsheet.GetNumberCols() < numcols:
                self.spreadsheet.SetNumberCols(numcols)
            colnum = 0
            for cell in row:
                self.spreadsheet.SetCellValue(rownum, colnum, str(cell))
                colnum = colnum + 1
            rownum = rownum + 1

    def on_plot(self, evt):
        '''Handles the plot button event, plots selected columns'''
        podx, pody = self.find_POD_data()
        hitmissx, hitmissy = self.find_hitmiss_data()
        bad_pod_data = (podx == []) or (pody == []) or (len(podx) != len(pody))
        bad_hitmiss_data = (hitmissx == []) or (hitmissy == []) or (len(hitmissx) != len(hitmissy))
        if not bad_pod_data and not bad_hitmiss_data:
            self.plot_data([podx, pody], [hitmissx, hitmissy])                 
        else:
            err_dlg = wx.MessageDialog(parent=self, message="This sheet does not appear to contain POD Data.",
                caption="Unable To Plot POD Data", style=wx.ICON_EXCLAMATION)
            err_dlg.ShowModal()

    def find_POD_data(self):
        '''Attempts to find the POD data in the active sheet.'''
        POD_xlbl = "CRACK LENGTH"
        POD_ylbl = "PRED.POD"
        x_axis = []
        y_axis = []
        POD_x, POD_y = self.find_data(POD_xlbl, POD_ylbl)
        if POD_x != -1 and POD_y != -1:
            x_axis = [cell for cell in self.xl_converter.get_col(self.sheet_choice.GetStringSelection(), POD_x)
                if isinstance(cell, float)]
            y_axis = [cell for cell in self.xl_converter.get_col(self.sheet_choice.GetStringSelection(), POD_y)
                if isinstance(cell, float)]
        return x_axis, y_axis

    def find_hitmiss_data(self):
        '''Attempts to find the hit/miss data in the active sheet.'''
        hitmiss_xlbl = "ACT. CRK LEN."
        hitmiss_ylbl = "HIT/MISS DATA"
        x_axis = []
        y_axis = []
        hitmiss_x, hitmiss_y = self.find_data(hitmiss_xlbl, hitmiss_ylbl)
        if hitmiss_x != -1 and hitmiss_y != -1:
            x_axis = [cell for cell in self.xl_converter.get_col(self.sheet_choice.GetStringSelection(), hitmiss_x)
                if isinstance(cell, float)]
            y_axis = [cell for cell in self.xl_converter.get_col(self.sheet_choice.GetStringSelection(), hitmiss_y)
                if isinstance(cell, float)]
        return x_axis, y_axis

    def find_data(self, col_label1, col_label2, rows_to_search=10):
        '''Searches the active sheet for two adjacent columns with the specified text.
        Defaults to search the first ten rows.  Returns the match as a tuple, or -1 for both
        elements if no match was found.
        '''
        column1 = -1
        column2 = -1
        for rownum in xrange(rows_to_search+1):
            row = self.xl_converter.get_row(self.sheet_choice.GetStringSelection(), rownum)
            for cell_no in xrange(len(row)):
                if row[cell_no] == col_label1 and row[cell_no+1] == col_label2:
                    column1 = cell_no
                    column2 = cell_no + 1
                    break
        return column1, column2

    def plot_data(self, pod_data, hitmiss_data):
        '''Opens a PlotPanel to display the POD results and the hit/miss data if found'''
        plot = plot_panel.PlotPanel(parent=self, pod_data=pod_data, hm_data=hitmiss_data, fname=self.xl_file)
        plot.Show()
