#!/usr/bin/env python

'''
convert_xl.py - reads and optionally exports simple Excel spreadsheets as ASCII delimited text files
Chris R. Coughlin (TRI/Austin, Inc.)

Requires xlrd module (http://www.lexicon.net/sjmachin/xlrd.htm).
As per xlrd, currently supports Excel versions 2003, 2002, XP, 2000, 97, 95, 5.0, 4.0, 3.0.
Does not support Excel 2007 as of yet.

License:  BSD
'''

import xlrd
import os.path

class ConvertExcelFile(object):
    '''Converts an Excel spreadsheet to a text file'''

    def __init__(self, xl_file, delim=','):
        self.xlfile = xl_file
        self.delim = delim
        self.workbook = self.get_workbook(self.xlfile)

    def get_workbook(self, fname):
        '''Gets the Excel workbook from the specified fname.
        Raises IOError if file not found.'''
        if not os.path.exists(fname):
            raise IOError("File not found")
        workbook = xlrd.open_workbook(fname)
        return workbook

    def read_sheet(self, sheet_name):
        '''Gets the contents of the specified Excel spreadsheet's sheet'''
        sheet = self.workbook.sheet_by_name(sheet_name)
        for row in xrange(sheet.nrows):
            yield sheet.row_values(row)

    def get_col(self, sheet_name, col_num):
        '''Returns the specified column from the sheet'''
        sheet = self.workbook.sheet_by_name(sheet_name)
        col_data = []
        if col_num < sheet.ncols:
            col_data = sheet.col_values(col_num)
        return col_data

    def get_row(self, sheet_name, row_num):
        '''Returns the specified row from the sheet'''
        sheet = self.workbook.sheet_by_name(sheet_name)
        row_data = []
        if row_num < sheet.nrows:
            row_data = sheet.row_values(row_num)
        return row_data

    def convert_sheet(self, sheet_name, output_fn):
        '''Retrieves the specified sheet name from the Excel spreadsheet and
        exports it to output_fn.'''
        sheet_contents = self.read_sheet(sheet_name)
        with open(output_fn, 'wb') as fidout:
            header_line = ' '.join(['# Sheet:', sheet_name, '\n'])
            fidout.write(header_line)
            for row in sheet_contents:
                row_str = [str(cell) for cell in row]
                fidout.write(self.delim.join(row_str))
                fidout.write('\n')

    def read_workbook(self):
        '''Gets the contents of the specified Excel spreadsheet, returns a list of sheet names
        and the contents.'''
        sheet_names = []
        contents = []
        sheet_names = self.workbook.sheet_names()
        for sheet in sheet_names:
            contents.append(self.read_sheet(sheet))
        return sheet_names, contents

    def convert_workbook(self, output_fn):
        '''Retrieves the entire Excel spreadsheet and
        exports it to output_fn.'''
        (sheet_names, workbook_contents) = self.read_workbook()
        with open(output_fn, 'wb') as fidout:
            sheet_num = 0
            for sheet in workbook_contents:
                sheet_name = sheet_names[sheet_num]
                header_line = ' '.join(['# Sheet:', sheet_name, '\n'])
                fidout.write(header_line)
                for row in sheet:
                    row_str = [str(cell) for cell in row]
                    fidout.write(self.delim.join(row_str))
                    fidout.write('\n')
                sheet_num += 1

def parse_args():
    '''Wrapper for handling command-line arguments.  Supports Python 2.7's argparse module and
    the Python 2.6 and under optparse module.'''
    try:
        import argparse
        argparser = argparse.ArgumentParser(description='Converts specified file to CSV.')
        argparser.add_argument('-f', '--file', dest='filename', action='store', help='Specifies file to open')
        argparser.add_argument('-s', '--sheetname', dest='sheetname', action='store', 
            help='Specifies sheet to convert')
        argparser.add_argument('-a', '--all', dest='all', action='store_true', help='Convert the entire Excel file')
        argparser.add_argument('-o', '--output', dest='outputfn', action='store', help='Specifies output filename')
        argparser.add_argument('-d', '--delim', dest='delim', action='store', default=',',
            help='Specifies the delimiter to use in the converted file (defaults to comma)')
        args = argparser.parse_args()
        return args
    except ImportError:  # Support Python 2.3 - 2.6
        import optparse
        argparser = optparse.OptionParser()
        argparser.add_option('-f', '--file', dest='filename', action='store', help='Specifies file to open')
        argparser.add_option('-s', '--sheetname', dest='sheetname', action='store', help='Specifies sheet to convert')
        argparser.add_option('-o', '--output', dest='outputfn', action='store', help='Specifies output filename')
        argparser.add_option('-a', '--all', dest='all', action='store_true', help='Convert the entire Excel file')
        argparser.add_option('-d', '--delim', dest='delim', action='store', default=',',
            help='Specifies the delimiter to use in the converted file (defaults to comma)')
        (options, args) =  argparser.parse_args()
        return options

def main():
    '''Main entry point for the module, exports specified Excel sheet/workbook as ASCII text.
    Usage:  convert_xl -fpath_to_file [-ssheetname]  [-a] [-ddelim] -ooutputfile

    To export a single sheet, specify the sheet's name e.g. -s"Sample 1" to export only the Sample 1 sheet.
    To export the entire spreadsheet, use -a.  Sheets are denoted by '#' in the output file.
    '''
    arguments = parse_args()
    file_name = arguments.filename
    output_filename = arguments.outputfn
    delim = arguments.delim
    sheet_name = arguments.sheetname
    convert_wb = arguments.all
    try:
        xl_converter = ConvertExcelFile(file_name, delim=delim)
        if not convert_wb:
            xl_converter.convert_sheet(output_fn=output_filename, sheet_name=sheet_name)
        else:
            xl_converter.convert_workbook(output_fn=output_filename)
    except IOError:  # File not found
        print("Excel file '{0}' not found.".format(file_name))
    except TypeError:  # No output and/or input file specified
        print("Please specify both an output and an input file.")

if __name__ == "__main__":
    main()