#!/usr/bin/env python

'''finddata.py - locates POD Data files (PODBrowser project)
Chris R. Coughlin (TRI/Austin, Inc.)
'''

import os
import os.path

class DataLocations(object):
    '''Specifies the default locations of the POD Data sets'''

    def __init__(self, basepath, methods=None):
        self.data_folder = str(basepath)
        if methods is not None:
            self.methods = list(methods)
        else:
            # Scan the data folder for first-level folders, assume these are the NDE methods
            cwd = os.getcwd() # Temporarily switch folders for os.listdir call
            os.chdir(self.data_folder)
            self.methods = [entry for entry in os.listdir('.') if os.path.isdir(entry)]
            os.chdir(cwd)
        self.folders = {}
        self.populate_folders()

    def populate_folders(self):
        '''Populates a dict with the inspection methods and their associated data folders'''
        for method in self.methods:
            self.folders[method] = os.path.join(self.data_folder, method)

    def method_folder(self, method_name):
        '''Returns the full path name of the specified method's data'''
        return self.folders.get(method_name, None)
        
class Data(object):
    ''' Walks a POD Handbook data folder and populates lists of data'''
    def __init__(self, base_folder):
        self.folder = base_folder
        self.inspections = []
        self.datasets = {}

    def populate(self):
        '''Finds the inspections and their respective data for a given POD data folder'''
        self.populate_inspections()
        self.populate_data()

    def populate_inspections(self):
        '''Finds the inspections for a POD data folder'''
        for root, dirs, files in os.walk(self.folder):
            for dname in dirs:
                dpath = os.path.join(root, dname)
                self.inspections.append(dpath)
    
    def populate_data(self):
        '''Populates the self.datasets dict with the name of the inspection and a list
        of the dataset filenames found associated with that inspection'''
        for inspection in self.inspections:
            inspection_data = []
            for root, dirs, files in os.walk(inspection):
                for fname in files:
                    inspection_data.append(os.path.join(root, fname))
            self.datasets[os.path.relpath(inspection, self.folder)] = inspection_data

    def get_data_fname(self, inspection_name, base_data_fname):
        '''Returns the full path to an inspection's data file based on the supplied basename, or None if not found.'''
        try:
            not_string = not(isinstance(inspection_name, str) or isinstance(inspection_name, unicode))
            if not_string:
                inspection_name = os.path.sep.join(inspection_name)
            data_basenames = [os.path.basename(data) for data in self.datasets[inspection_name]]
            return self.datasets[inspection_name][data_basenames.index(base_data_fname)]
        except (KeyError, ValueError): # Inspection not in the list of data, data file not found (respectively)
            return None
    
if __name__ == "__main__":
    # Example of usage
    dataloc = DataLocations('/home/ccoughlin/src/Python/podbrowser/data')    
    mp = Data(dataloc.method_folder("Magnetic Particle"))
    mp.populate()
    print("Mag Particle inspections:")
    for insp in mp.inspections:
        print("\t{0}".format(insp))
    print("Mag Particle Data sets:")
    for inspection, dataset in mp.datasets.items():
        print("\t{0} = {1}".format(inspection, dataset))