#!/usr/bin/env python

'''
open_file.py - opens a file with the associated application (Windows, OS X, POSIX)
Chris R. Coughlin (TRI/Austin, Inc.)

Based on discussion and code from StackOverflow:  
http://stackoverflow.com/questions/434597/open-document-with-default-application-in-python

License:  BSD
'''

import subprocess
import os
import os.path
import warnings

def open_file(filename):
    '''Opens the file filename with its associated application.
    Windows, POSIX, OS X supported directly; other OSes may
    also work but function will issue a warning.

    Exceptions:
    IOError - file not found
    WindowsError - no associated application for this filetype, application not found (Windows only)
    OSError - application not found
    '''
    if not os.path.exists(filename):
        raise IOError("File not found")
    if os.name == 'nt':
        os.startfile(filename)
    else:
        if os.name == 'mac':
            cmd = 'open'
        elif os.name == 'posix':
            cmd = 'xdg-open'
        else:
            warnings.warn('Possibly unsupported OS')
            cmd = 'open'
        try:
            subprocess.check_call((cmd, filename))
        except subprocess.CalledProcessError:  # Open return code wasn't zero, redirect to an OSError
            raise OSError("Unable to open file with associated application")

def main():
    '''Command-line interface to open_file
    Usage:  open_file -fpath_to_file
    '''
    try:
        import argparse
        argparser = argparse.ArgumentParser(description='Runs specified file with its associated application.')
        argparser.add_argument('-f', '--file', dest='filename', action='store', help='Specifies file to open')
        args = argparser.parse_args()
        file_name = args.filename
    except ImportError:  # Support Python 2.6
        import optparse
        argparser = optparse.OptionParser()
        argparser.add_option('-f', '--file', dest='filename', action='store', help='Specifies file to open')
        (options, args) =  argparser.parse_args()
        file_name = options.filename
    try:
        open_file(file_name)
    except IOError: # File not found
        print("File '{0}' was not found.".format(file_name))
    except (OSError, WindowsError): # No associated application
        print("No associated application with this filetype.")    

if __name__ == "__main__":
    main()